package com.company.mystorage.view.attachments;

import com.company.mystorage.entity.Attachments;

import com.company.mystorage.view.AppFileStorageUploadField;
import com.company.mystorage.view.main.MainView;

import com.vaadin.flow.router.Route;
import io.jmix.core.FileRef;
import io.jmix.core.FileStorageException;
import io.jmix.flowui.component.upload.FileStorageUploadField;
import io.jmix.flowui.component.upload.receiver.FileTemporaryStorageBuffer;
import io.jmix.flowui.kit.component.upload.event.FileUploadSucceededEvent;
import io.jmix.flowui.sys.registration.ComponentRegistrationBuilder;
import io.jmix.flowui.upload.TemporaryStorage;
import io.jmix.flowui.view.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.UUID;

import io.jmix.flowui.sys.registration.ComponentRegistration;

import com.company.mystorage.MyFileStorage;
import org.springframework.context.annotation.Bean;

@Route(value = "attachmentses/:id", layout = MainView.class)
@ViewController("Attachments.detail")
@ViewDescriptor("attachments-detail-view.xml")
@EditedEntityContainer("attachmentsDc")
public class AttachmentsDetailView extends StandardDetailView<Attachments>
{
	@ViewComponent
	private FileStorageUploadField attachmentField;

	private Path getPath() {
		Path root = Path.of("");
		return Path.of(root.toAbsolutePath().toString(), "Attachments");
	}

	@Autowired
	private TemporaryStorage temporaryStorage;

	@Subscribe("attachmentField")
	public void onAttachmentFieldFileUploadSucceeded(final FileUploadSucceededEvent<FileStorageUploadField> event) {
		if (event.getReceiver() instanceof FileTemporaryStorageBuffer buffer) {
			UUID fileId = buffer.getFileData().getFileInfo().getId();
			File file = temporaryStorage.getFile(fileId);
			if (file != null) {
				try {
					//  the file is stored in the temporaryStorage
					FileRef fileRef;
					//
					String fileName = getPath() + File.separator + event.getFileName();

					// Special handling for NTFS ADS: Don't accept colon in the fileName.
					//fileName = fileName.substring(fileName.indexOf(':')+1);

					try (InputStream io = new FileInputStream(file)) {
						Path path = getPath();
						//  we have access to the inputfile
						Files.createDirectories(path);
						MyFileStorage fileStorage = new MyFileStorage("myStorage", getPath().toString());
						fileRef = fileStorage.saveStream(fileName, io);
						//attachmentField.setValue(FileRef.fromString(event.getFileName()));
						attachmentField.setValue(fileRef);
					} catch (FileNotFoundException e) {
						throw new FileStorageException(FileStorageException.Type.FILE_NOT_FOUND, "Temp file not found " + file.getAbsolutePath());
					} catch (IOException e) {
						throw new FileStorageException(FileStorageException.Type.IO_EXCEPTION, fileName);
					}

				} catch (Exception e) {
					throw new RuntimeException(e);
				}

			}
		}
	}
}